`ifdef RTAVR_SMP
/*
 * Copyright (C) 2011 Koji Suzuki
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.1 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

`include "rtavr_defs.v"

// global defines : 

//
// Module Name: rtavr_smp
//   ver 0.9.0
// 
// Target Device:   xc3s200a-4ft256
// Product Version: ISE 12.4
// Design Goal:     Balanced
// 

module rtavr_smp (
		input RESET
	`ifdef SYS_DRIVEN_CLK4PH
		, input CLK_0
		, input CLK_90
		, input CLK_180
		, input CLK_270
	`else // SYS_DRIVEN_CLK2X
		, input CLK2X
		, output CLK_OUT
	`endif

		// external I/O
  `ifdef RTAVR_PORT_ESCALATION
		, input [23:0] PIN
		, output [23:0] DDR
		, output [23:0] PORT
  `else
    `ifdef IOR_HAVE_PORTA
		, inout [7:0] PORTA
    `endif
    `ifdef IOR_HAVE_PORTB
		, inout [7:0] PORTB
    `endif
    `ifdef IOR_HAVE_PORTC
		, inout [7:0] PORTC
    `endif
  `endif
  `ifdef IOR_HAVE_CPI1
		// external I/O
		, output [7:0] CPI1_TXD
		, output CPI1_TX
		, input CPI1_TX_ACK
		, output CPI1_IXS
		, input CPI1_IXS_ACK

		, input [7:0] CPI1_RXD
		, input CPI1_RX
		, output CPI1_RX_ACK
		, input CPI1_IXR
		, output CPI1_IXR_ACK

  `endif
		, output PW_SLEEP
  `ifdef PRC_WATCH_SMP
		, output PW_SLEEP_P1
  `endif
  `ifdef PRC_WATCH_WDR
		, output PW_WDR
    `ifdef PRC_WATCH_SMP
		, output PW_WDR_P1
    `endif
  `endif
  `ifdef PRC_WATCH
		, output PW_STORE
		, output PW_LOAD
		, output [11:0] PW_PC
		, output [7:0] PW_SREG
		, output [11:0] PW_SP
		, output [7:0] PW_LOAD_REG
		, output [3:0] PW_LOAD_ID
		, output [7:0] PW_STORE_MEM
		, output [11:0] PW_STORE_ADDR
    `ifdef PRC_WATCH_SMP
		, output PW_STORE_P1
		, output PW_LOAD_P1
		, output [11:0] PW_PC_P1
		, output [7:0] PW_SREG_P1
		, output [11:0] PW_SP_P1
		, output [7:0] PW_LOAD_REG_P1
		, output [3:0] PW_LOAD_ID_P1
		, output [7:0] PW_STORE_MEM_P1
		, output [11:0] PW_STORE_ADDR_P1
    `endif
  `endif
`ifdef SUPPORT_ISP
		, input ISP_STORE
		, input ISP_LOAD
		, input [15:0] ISP_ADDR
		, input [7:0] ISP_STORE_DATA
		, output [7:0] ISP_LOAD_DATA
`endif
		);

  `ifdef RTAVR_PORT_ESCALATION
  `else
		wire [23:0] PIN;
		wire [23:0] DDR;
		wire [23:0] PORT;
  `endif

	wire [11:0] ROM_ADDRA_P0;
	wire [15:0] ROM_DOA_P0;
	wire [12:0] ROM_ADDRB_P0;
	wire [7:0]  ROM_DOB_P0;
	wire [7:0]  ROM_DIB_P0; 

	wire [11:0] ROM_ADDRA_P1;
	wire [15:0] ROM_DOA_P1;
	wire [12:0] ROM_ADDRB_P1;
	wire [7:0]  ROM_DOB_P1;
	wire [7:0]  ROM_DIB_P1; 

	wire [12:0] RAM_ADDRA;
	wire [7:0]  RAM_DOA;
	wire [7:0]  RAM_DIA; 

	wire [12:0] RAM_ADDRB;
	wire [7:0]  RAM_DOB;
	wire [7:0]  RAM_DIB;   
  
  rtavr_rom #(.SIZE(`ROM_SIZE))
	ROM (
		// PORTA
		.OEA(ROM_OEA_P0)
		, .ADDRA(ROM_ADDRA_P0[11:0])
		, .DOA(ROM_DOA_P0[15:0])
		// PORTB
		, .OEB(ROM_OEB_P0)
		, .WEB(ROM_WEB_P0)
		, .ADDRB(ROM_ADDRB_P0[12:0])
		, .DIB(ROM_DIB_P0[7:0])
		, .DOB(ROM_DOB_P0[7:0])
		// PORTC
		, .OEC(ROM_OEA_P1)
		, .ADDRC(ROM_ADDRA_P1[11:0])
		, .DOC(ROM_DOA_P1[15:0])
		// PORTD
		, .OED(ROM_OEB_P1)
		, .WED(ROM_WEB_P1)
		, .ADDRD(ROM_ADDRB_P1[12:0])
		, .DID(ROM_DIB_P1[7:0])
		, .DOD(ROM_DOB_P1[7:0])
    `ifdef SYS_DRIVEN_CLK4PH
		, .CLK_0(CLK_0)
		, .CLK_90(CLK_90)
		, .CLK_180(CLK_180)
		, .CLK_270(CLK_270)
     `else // SYS_DRIVEN_CLK2X
		, .CLK2X(CLK2X)
		, .CLK(CLK)
    `endif
		);

  rtavr_sram #(.SIZE(`RAM_SIZE)) RAM (
		  .WEA(RAM_WEA)
		, .ADDRA(RAM_ADDRA[12:0])
		, .DIA(RAM_DIA[7:0]), .DOA(RAM_DOA[7:0])

		, .WEB(RAM_WEB)
		, .ADDRB(RAM_ADDRB[12:0])
		, .DIB(RAM_DIB[7:0]), .DOB(RAM_DOB[7:0])
`ifdef SYS_DRIVEN_CLK4PH
		, .CLK_0(CLK_0)
		, .CLK_90(CLK_90)
		, .CLK_180(CLK_180)
		, .CLK_270(CLK_270)
`else // SYS_DRIVEN_CLK2X
		, .CLK2X(CLK2X)
		, .CLK(CLK)
`endif
		);

	wire [5:0]  IOR_ADDRB;
	wire [7:0]  IOR_DIB;
	wire [7:0]  IOR_DOB;
	wire [7:0]  FLAGS_OUT;
	wire [7:0]  FLAGS_IN;
	wire [15:0] SP_OUT;
	wire [15:0] SP_IN;

	wire [5:0]  IOR_ADDRB_P1;
	wire [7:0]  IOR_DIB_P1;
	wire [7:0]  IOR_DOB_P1;
	wire [7:0]  FLAGS_OUT_P1;
	wire [7:0]  FLAGS_IN_P1;
	wire [15:0] SP_OUT_P1;
	wire [15:0] SP_IN_P1;
//  `ifdef IOR_HAVE_CPI1
//	wire [7:0]  CPI1_TXD;
//	wire [7:0]  CPI1_RXD;
//  `endif
`ifdef SUPPORT_LOCKING
	wire  [4:0] CPU0_LK_ADDR;
	wire  [4:0] CPU1_LK_ADDR;
   `ifdef IOR_DUALPORT
   `else
	wire  CPU0_LK_NAK = 1'b0;
   `endif
`endif
  `ifdef IOR_HAVE_RAMWIN
	wire RAMW_WE;
	wire RAMW_OE;
	wire [12:0] RAMW_ADDR;
	wire [7:0] RAMW_DO;
    `ifdef IOR_DUALPORT
	wire RAMW_WE_P1;
	wire RAMW_OE_P1;
	wire [12:0] RAMW_ADDR_P1;
	wire [7:0] RAMW_DO_P1;
    `endif
  `endif

  rtavr_ior IOR (.RESET(RESET)
    `ifdef SYS_DRIVEN_CLK4PH
		, .CLK_0(CLK_0)
		, .CLK_90(CLK_90)
		, .CLK_180(CLK_180)
		, .CLK_270(CLK_270)
    `else // SYS_DRIVEN_CLK2X
		, .CLK2X(CLK2X)
		, .CLK(CLK)
    `endif
  `ifdef IOR_PORT_ESCALATION
		, .DDR(DDR[23:0])
		, .PORT(PORT[23:0])
		, .PIN(PIN[23:0])
  `else
    `ifdef IOR_HAVE_PORTA
		, .PORTA(PORTA[7:0])
    `endif
    `ifdef IOR_HAVE_PORTB
		, .PORTB(PORTB[7:0])
    `endif
    `ifdef IOR_HAVE_PORTC
		, .PORTC(PORTC[7:0])
    `endif
  `endif
		, .ADDR(IOR_ADDRB[5:0]), .DI(IOR_DIB[7:0]), .DO(IOR_DOB[7:0])
		, .RD(IOR_OE), .WR(IOR_WEB)
  `ifdef IOR_HAVE_INT0
		, .INT0(IOR_INT0)
		, .RST_INT0(IOR_RST_INT0)
    `ifdef IOR_HAVE_INT1
		, .INT1(IOR_INT1)
		, .RST_INT1(IOR_RST_INT1)
    `endif
  `endif
		// SREG(FLAGS) & SP
		, .FLAGS_OUT(FLAGS_OUT[7:0]) , .FLAGS_IN(FLAGS_IN[7:0])
		, .SP_OUT(SP_OUT[15:0]) , .SP_IN(SP_IN[15:0])
  `ifdef IOR_HAVE_RAMWIN
		, .RAMW_WE(RAMW_WE)
		, .RAMW_OE(RAMW_OE)
		, .RAMW_ADDR(RAMW_ADDR)
		, .RAMW_DO(RAMW_DO)
  `endif
  `ifdef IOR_HAVE_NVM
		, .ROM_WP(ROM_WP)
		, .RAM_WP(RAM_WP)
  `endif
  `ifdef IOR_HAVE_TIM0
		// INTERRUPTS 
		, .TIM0_OVF(IOR_TIM0_OVF)
		, .RST_TIM0_OVF(IOR_RST_TIM0_OVF)
		, .TIM0_COMPA(IOR_TIM0_COMPA)
		, .RST_TIM0_COMPA(IOR_RST_TIM0_COMPA)
		, .TIM0_COMPB(IOR_TIM0_COMPB)
		, .RST_TIM0_COMPB(IOR_RST_TIM0_COMPB)
  `endif
  `ifdef IOR_HAVE_SPI
		, .SPI_INT(IOR_SPI_INT)
		, .RST_SPI_INT(IOR_RST_SPI_INT)
  `endif
  `ifdef IOR_HAVE_USART
		, .USART_TX_INT(IOR_USART_TX)
		, .USART_UDRE_INT(IOR_USART_UDRE)
		, .USART_RX_INT(IOR_USART_RX)
		, .RST_USART_TX_INT(IOR_RST_USART_TX)
  `endif
  `ifdef IOR_HAVE_TWI
		, .TWI_SLAVE(IOR_TWI_SLAVE)
		, .RST_TWI_SLAVE(IOR_RST_TWI_SLAVE)
  `endif
  `ifdef IOR_HAVE_CPI0
		// interrupt
		, .CPI0_INT(IOR_CPI0_INT)
		, .RST_CPI0_INT(IOR_RST_CPI0_INT)
  `endif
  `ifdef IOR_HAVE_CPI1
		// external I/O
		, .CPI1_TXD(CPI1_TXD[7:0])
		, .CPI1_RXD(CPI1_RXD[7:0])
		, .CPI1_TX(CPI1_TX)
		, .CPI1_TX_ACK(CPI1_TX_ACK)
		, .CPI1_RX(CPI1_RX)
		, .CPI1_RX_ACK(CPI1_RX_ACK)
		, .CPI1_IXR(CPI1_IXR)
		, .CPI1_IXR_ACK(CPI1_IXR_ACK)
		, .CPI1_IXS(CPI1_IXS)
		, .CPI1_IXS_ACK(CPI1_IXS_ACK)

		// interrupt
		, .CPI1_INT(IOR_CPI1_INT)
		, .RST_CPI1_INT(IOR_RST_CPI1_INT)
  `endif
`ifdef IOR_DUALPORT
   `ifdef SUPPORT_LOCKING
		, .CPU0_LK_REQ(CPU0_LK_REQ)
		, .CPU0_LK_REQ2X(CPU0_LK_REQ2X1)
		, .CPU0_LK_NAK(CPU0_LK_NAK)
		, .CPU0_LK_ADDR(CPU0_LK_ADDR)
		, .CPU1_LK_REQ(CPU1_LK_REQ)
		, .CPU1_LK_REQ2X(CPU1_LK_REQ2X1)
		, .CPU1_LK_NAK(CPU1_LK_NAK)
		, .CPU1_LK_ADDR(CPU1_LK_ADDR)
   `endif
		, .CPU1_ADDR(IOR_ADDRB_P1[5:0])
		, .CPU1_DI(IOR_DIB_P1[7:0]), .CPU1_DO(IOR_DOB_P1[7:0])
		, .CPU1_RD(IOR_OE_P1), .CPU1_WR(IOR_WEB_P1)
  `ifdef IOR_HAVE_INT0
		, .CPU1_INT0(IOR_INT0_P1)
		, .CPU1_RST_INT0(IOR_RST_INT0_P1)
    `ifdef IOR_HAVE_INT1
		, .CPU1_INT1(IOR_INT1_P1)
		, .CPU1_RST_INT1(IOR_RST_INT1_P1)
    `endif
  `endif
		// SREG(FLAGS) & SP
		, .CPU1_FLAGS_OUT(FLAGS_OUT_P1[7:0]) , .CPU1_FLAGS_IN(FLAGS_IN_P1[7:0])
		, .CPU1_SP_OUT(SP_OUT_P1[15:0]) , .CPU1_SP_IN(SP_IN_P1[15:0])
  `ifdef IOR_HAVE_RAMWIN
		, .CPU1_RAMW_WE(RAMW_WE_P1)
		, .CPU1_RAMW_OE(RAMW_OE_P1)
		, .CPU1_RAMW_ADDR(RAMW_ADDR_P1)
		, .CPU1_RAMW_DO(RAMW_DO_P1)
  `endif
  `ifdef IOR_HAVE_NVM
		, .CPU1_ROM_WP(ROM_WP_P1)
		, .CPU1_RAM_WP(RAM_WP_P1)
  `endif
  `ifdef IOR_HAVE_TIM0
		// INTERRUPTS 
		, .CPU1_TIM0_OVF(IOR_TIM0_OVF_P1)
		, .CPU1_RST_TIM0_OVF(IOR_RST_TIM0_OVF_P1)
		, .CPU1_TIM0_COMPA(IOR_TIM0_COMPA_P1)
		, .CPU1_RST_TIM0_COMPA(IOR_RST_TIM0_COMPA_P1)
		, .CPU1_TIM0_COMPB(IOR_TIM0_COMPB_P1)
		, .CPU1_RST_TIM0_COMPB(IOR_RST_TIM0_COMPB_P1)
  `endif
  `ifdef IOR_HAVE_SPI
		, .CPU1_SPI_INT(IOR_SPI_INT_P1)
		, .CPU1_RST_SPI_INT(IOR_RST_SPI_INT_P1)
  `endif
  `ifdef IOR_HAVE_USART
		, .CPU1_USART_TX_INT(IOR_USART_TX_P1)
		, .CPU1_USART_UDRE_INT(IOR_USART_UDRE_P1)
		, .CPU1_USART_RX_INT(IOR_USART_RX_P1)
		, .CPU1_RST_USART_TX_INT(IOR_RST_USART_TX_P1)
  `endif
  `ifdef IOR_HAVE_TWI
		, .CPU1_TWI_SLAVE(IOR_TWI_SLAVE_P1)
		, .CPU1_RST_TWI_SLAVE(IOR_RST_TWI_SLAVE_P1)
  `endif
  `ifdef IOR_HAVE_CPI0
		// interrupt
		, .CPU1_CPI0_INT(IOR_CPI0_INT_P1)
		, .CPU1_RST_CPI0_INT(IOR_RST_CPI0_INT_P1)
  `endif
  `ifdef IOR_HAVE_CPI1
		// interrupt
		, .CPU1_CPI1_INT(IOR_CPI1_INT_P1)
		, .CPU1_RST_CPI1_INT(IOR_RST_CPI1_INT_P1)
  `endif
`endif // IOR_DUALPORT
		);

`ifdef PRC_WATCH
	wire [7:0]  PW_SREG_P0;
	wire [11:0] PW_SP_P0;
	wire [11:0] PW_PC_P0;
	wire [7:0]  PW_LOAD_REG_P0;
	wire [3:0]  PW_LOAD_ID_P0;
	wire [7:0]  PW_STORE_MEM_P0;
	wire [11:0] PW_STORE_ADDR_P0;
`endif

  rtavr #(.CPUID(0)) RTAVR0 (
		.RESET(RESET)
    `ifdef SYS_DRIVEN_CLK4PH
		, .CLK_0(CLK_0)
		, .CLK_90(CLK_90)
		, .CLK_180(CLK_180)
		, .CLK_270(CLK_270)
    `else // SYS_DRIVEN_CLK2X
		, .CLK2X(CLK2X)
		, .CLK_OUT(CLK_OUT)
    `endif
   `ifdef SUPPORT_LOCKING
		, .IOR_LK_REQ(CPU0_LK_REQ)
		, .IOR_LK_REQ2X(CPU0_LK_REQ2X1)
		, .IOR_LK_NAK(CPU0_LK_NAK)
		, .IOR_LK_ADDR(CPU0_LK_ADDR)
   `endif
		,  .PW_SLEEP(PW_SLEEP_P0)
`ifdef PRC_WATCH_WDR
		,  .PW_WDR(PW_WDR_P0)
`endif
`ifdef PRC_WATCH
		,  .PW_STORE(PW_STORE_P0)
		,  .PW_LOAD(PW_LOAD_P0)
		,  .PW_SREG(PW_SREG_P0[7:0])
		,  .PW_SP(PW_SP_P0[11:0])
		,  .PW_PC(PW_PC_P0[11:0])
		,  .PW_LOAD_REG(PW_LOAD_REG_P0[7:0])
		,  .PW_LOAD_ID(PW_LOAD_ID_P0[3:0])
		,  .PW_STORE_MEM(PW_STORE_MEM_P0[7:0])
		,  .PW_STORE_ADDR(PW_STORE_ADDR_P0[11:0])
`endif
`ifdef SUPPORT_ISP
		,  .ISP_STORE(ISP_STORE)
		,  .ISP_LOAD(ISP_LOAD)
		,  .ISP_ADDR(ISP_ADDR[15:0])
		,  .ISP_STORE_DATA(ISP_STORE_DATA[7:0])
		,  .ISP_LOAD_DATA(ISP_LOAD_DATA[7:0])
`endif
		, .ROM_OEA(ROM_OEA_P0)
		, .ROM_ADDRA(ROM_ADDRA_P0[11:0])
		, .ROM_DOA(ROM_DOA_P0[15:0])

		, .ROM_OEB(ROM_OEB_P0)
		, .ROM_WEB(ROM_WEB_P0)
		, .ROM_ADDRB(ROM_ADDRB_P0[12:0])
		, .ROM_DIB(ROM_DIB_P0[7:0])
		, .ROM_DOB(ROM_DOB_P0[7:0])

		, .RAM_WEB(RAM_WEB)
		, .RAM_ADDRB(RAM_ADDRB[12:0])
		, .RAM_DIB(RAM_DIB[7:0])
		, .RAM_DOB(RAM_DOB[7:0])
  `ifdef OUTPUT_MUX_NONE
  `else
		, .RAM_OEB(RAM_OEB)
  `endif
		// IOR
		, .IOR_ADDRB(IOR_ADDRB[5:0])
		, .IOR_DIB(IOR_DIB[7:0]), .IOR_DOB(IOR_DOB[7:0])
		, .IOR_OE(IOR_OE), .IOR_WEB(IOR_WEB)
  `ifdef IOR_HAVE_INT0
		, .IOR_INT0(IOR_INT0)
		, .IOR_RST_INT0(IOR_RST_INT0)
    `ifdef IOR_HAVE_INT1
		, .IOR_INT1(IOR_INT1)
		, .IOR_RST_INT1(IOR_RST_INT1)
    `endif
  `endif
		// SREG(FLAGS) & SP
		, .FLAGS_IN(FLAGS_OUT[7:0]) , .FLAGS_OUT(FLAGS_IN[7:0])
		, .SP_IN(SP_OUT[15:0]) , .SP_OUT(SP_IN[15:0])
  `ifdef IOR_HAVE_RAMWIN
		, .RAMW_WE(RAMW_WE)
		, .RAMW_OE(RAMW_OE)
		, .RAMW_ADDR(RAMW_ADDR)
		, .RAMW_DO(RAMW_DO)
  `endif
  `ifdef IOR_HAVE_NVM
		, .ROM_WP(ROM_WP)
		, .RAM_WP(RAM_WP)
  `endif
  `ifdef IOR_HAVE_TIM0
		// INTERRUPTS 
		, .IOR_TIM0_OVF(IOR_TIM0_OVF)
		, .IOR_RST_TIM0_OVF(IOR_RST_TIM0_OVF)
		, .IOR_TIM0_COMPA(IOR_TIM0_COMPA)
		, .IOR_RST_TIM0_COMPA(IOR_RST_TIM0_COMPA)
		, .IOR_TIM0_COMPB(IOR_TIM0_COMPB)
		, .IOR_RST_TIM0_COMPB(IOR_RST_TIM0_COMPB)
  `endif
  `ifdef IOR_HAVE_SPI
		, .IOR_SPI_INT(IOR_SPI_INT)
		, .IOR_RST_SPI_INT(IOR_RST_SPI_INT)
  `endif
  `ifdef IOR_HAVE_USART
		, .IOR_USART_TX(IOR_USART_TX)
		, .IOR_USART_UDRE(IOR_USART_UDRE)
		, .IOR_USART_RX(IOR_USART_RX)
		, .IOR_RST_USART_TX(IOR_RST_USART_TX)
  `endif
  `ifdef IOR_HAVE_TWI
		, .IOR_TWI_SLAVE(IOR_TWI_SLAVE)
		, .IOR_RST_TWI_SLAVE(IOR_RST_TWI_SLAVE)
  `endif
  `ifdef IOR_HAVE_CPI0
		// interrupt
		, .IOR_CPI0_INT(IOR_CPI0_INT)
		, .IOR_RST_CPI0_INT(IOR_RST_CPI0_INT)
  `endif
  `ifdef IOR_HAVE_CPI1
		// interrupt
		, .IOR_CPI1_INT(IOR_CPI1_INT)
		, .IOR_RST_CPI1_INT(IOR_RST_CPI1_INT)
  `endif
		);

`ifdef SYS_DRIVEN_CLK4PH
`else // SYS_DRIVEN_CLK2X
	assign CLK = CLK_OUT;
`endif

`ifdef PRC_WATCH
   `ifdef PRC_WATCH_SMP
   `else
	wire [7:0]  PW_SREG_P1;
	wire [11:0] PW_SP_P1;
	wire [11:0] PW_PC_P1;
	wire [7:0]  PW_LOAD_REG_P1;
	wire [3:0]  PW_LOAD_ID_P1;
	wire [7:0]  PW_STORE_MEM_P1;
	wire [11:0] PW_STORE_ADDR_P1;
   `endif
`endif

  rtavr #(.CPUID(1)) RTAVR1 (
		.RESET(RESET)
    `ifdef SYS_DRIVEN_CLK4PH
		, .CLK_0(CLK_0)
		, .CLK_90(CLK_90)
		, .CLK_180(CLK_180)
		, .CLK_270(CLK_270)
    `else // SYS_DRIVEN_CLK2X
		, .CLK2X(CLK2X)
		, .CLK_OUT(CLK_OUT_P1)
    `endif
   `ifdef SUPPORT_LOCKING
		, .IOR_LK_REQ(CPU1_LK_REQ)
		, .IOR_LK_REQ2X(CPU1_LK_REQ2X1)
		, .IOR_LK_NAK(CPU1_LK_NAK)
		, .IOR_LK_ADDR(CPU1_LK_ADDR)
   `endif
		,  .PW_SLEEP(PW_SLEEP_P1)
`ifdef PRC_WATCH_WDR
		,  .PW_WDR(PW_WDR_P1)
`endif
`ifdef PRC_WATCH
		,  .PW_STORE(PW_STORE_P1)
		,  .PW_LOAD(PW_LOAD_P1)
		,  .PW_SREG(PW_SREG_P1[7:0])
		,  .PW_SP(PW_SP_P1[11:0])
		,  .PW_PC(PW_PC_P1[11:0])
		,  .PW_LOAD_REG(PW_LOAD_REG_P1[7:0])
		,  .PW_LOAD_ID(PW_LOAD_ID_P1[3:0])
		,  .PW_STORE_MEM(PW_STORE_MEM_P1[7:0])
		,  .PW_STORE_ADDR(PW_STORE_ADDR_P1[11:0])
`endif
`ifdef SUPPORT_ISP
		,  .ISP_STORE(1'b0)
		,  .ISP_LOAD(1'b0)
		,  .ISP_ADDR(16'b0)
		,  .ISP_STORE_DATA(8'b0)
`endif
		, .ROM_ADDRA(ROM_ADDRA_P1[11:0])
		, .ROM_DOA(ROM_DOA_P1[15:0])
		, .ROM_WEB(ROM_WEB_P1)
		, .ROM_ADDRB(ROM_ADDRB_P1[12:0])
		, .ROM_DIB(ROM_DIB_P1[7:0])
		, .ROM_DOB(ROM_DOB_P1[7:0])
		, .ROM_OEA(ROM_OEA_P1)
		, .ROM_OEB(ROM_OEB_P1)

		, .RAM_WEB(RAM_WEA)
		, .RAM_ADDRB(RAM_ADDRA[12:0])
		, .RAM_DIB(RAM_DIA[7:0])
		, .RAM_DOB(RAM_DOA[7:0])
  `ifdef OUTPUT_MUX_NONE
  `else
		, .RAM_OEB(RAM_OEA)
  `endif
		// IOR
		, .IOR_ADDRB(IOR_ADDRB_P1[5:0])
		, .IOR_DIB(IOR_DIB_P1[7:0]), .IOR_DOB(IOR_DOB_P1[7:0])
		, .IOR_OE(IOR_OE_P1), .IOR_WEB(IOR_WEB_P1)
  `ifdef IOR_HAVE_INT0
		, .IOR_INT0(IOR_INT0_P1)
		, .IOR_RST_INT0(IOR_RST_INT0_P1)
    `ifdef IOR_HAVE_INT1
		, .IOR_INT1(IOR_INT1_P1)
		, .IOR_RST_INT1(IOR_RST_INT1_P1)
    `endif
  `endif
		// SREG(FLAGS) & SP
		, .FLAGS_IN(FLAGS_OUT_P1[7:0]) , .FLAGS_OUT(FLAGS_IN_P1[7:0])
		, .SP_IN(SP_OUT_P1[15:0]) , .SP_OUT(SP_IN_P1[15:0])
  `ifdef IOR_HAVE_RAMWIN
		, .RAMW_WE(RAMW_WE_P1)
		, .RAMW_OE(RAMW_OE_P1)
		, .RAMW_ADDR(RAMW_ADDR_P1)
		, .RAMW_DO(RAMW_DO_P1)
  `endif
  `ifdef IOR_HAVE_NVM
		, .ROM_WP(ROM_WP_P1)
		, .RAM_WP(RAM_WP_P1)
  `endif
  `ifdef IOR_HAVE_TIM0
		// INTERRUPTS 
		, .IOR_TIM0_OVF(IOR_TIM0_OVF_P1)
		, .IOR_RST_TIM0_OVF(IOR_RST_TIM0_OVF_P1)
		, .IOR_TIM0_COMPA(IOR_TIM0_COMPA_P1)
		, .IOR_RST_TIM0_COMPA(IOR_RST_TIM0_COMPA_P1)
		, .IOR_TIM0_COMPB(IOR_TIM0_COMPB_P1)
		, .IOR_RST_TIM0_COMPB(IOR_RST_TIM0_COMPB_P1)
  `endif
  `ifdef IOR_HAVE_SPI
		, .IOR_SPI_INT(IOR_SPI_INT_P1)
		, .IOR_RST_SPI_INT(IOR_RST_SPI_INT_P1)
  `endif
  `ifdef IOR_HAVE_USART
		, .IOR_USART_TX(IOR_USART_TX_P1)
		, .IOR_USART_UDRE(IOR_USART_UDRE_P1)
		, .IOR_USART_RX(IOR_USART_RX_P1)
		, .IOR_RST_USART_TX(IOR_RST_USART_TX_P1)
  `endif
  `ifdef IOR_HAVE_TWI
		, .IOR_TWI_SLAVE(IOR_TWI_SLAVE_P1)
		, .IOR_RST_TWI_SLAVE(IOR_RST_TWI_SLAVE_P1)
  `endif
  `ifdef IOR_HAVE_CPI0
		// interrupt
		, .IOR_CPI0_INT(IOR_CPI0_INT_P1)
		, .IOR_RST_CPI0_INT(IOR_RST_CPI0_INT_P1)
  `endif
  `ifdef IOR_HAVE_CPI1
		// interrupt
		, .IOR_CPI1_INT(IOR_CPI1_INT_P1)
		, .IOR_RST_CPI1_INT(IOR_RST_CPI1_INT_P1)
  `endif
		);

   `ifdef PRC_WATCH_P1
	assign PW_SLEEP      = PW_SLEEP_P1; 
   `else
	assign PW_SLEEP      = PW_SLEEP_P0; 
   `endif
`ifdef PRC_WATCH_WDR
   `ifdef PRC_WATCH_P1
	assign PW_WDR        = PW_WDR_P1; 
   `else
	assign PW_WDR        = PW_WDR_P0; 
   `endif
`endif
`ifdef PRC_WATCH
   `ifdef PRC_WATCH_P1
	assign PW_STORE      = PW_STORE_P1; 
	assign PW_LOAD       = PW_LOAD_P1; 
	assign PW_SREG       = PW_SREG_P1; 
	assign PW_SP         = PW_SP_P1;
	assign PW_PC         = PW_PC_P1;
	assign PW_LOAD_REG   = PW_LOAD_REG_P1;
	assign PW_LOAD_ID    = PW_LOAD_ID_P1;
	assign PW_STORE_MEM  = PW_STORE_MEM_P1;
	assign PW_STORE_ADDR = PW_STORE_ADDR_P1;
   `else
	assign PW_STORE      = PW_STORE_P0; 
	assign PW_LOAD       = PW_LOAD_P0; 
	assign PW_SREG       = PW_SREG_P0; 
	assign PW_SP         = PW_SP_P0;
	assign PW_PC         = PW_PC_P0;
	assign PW_LOAD_REG   = PW_LOAD_REG_P0;
	assign PW_LOAD_ID    = PW_LOAD_ID_P0;
	assign PW_STORE_MEM  = PW_STORE_MEM_P0;
	assign PW_STORE_ADDR = PW_STORE_ADDR_P0;
   `endif
`endif

`ifdef RTAVR_PORT_ESCALATION
`elsif IOR_PORT_ESCALATION
  wire [23:0] PU;     // XXX not yet
  generate 
    genvar i;
  `ifdef IOR_HAVE_PORTA
    for (i=0; i<8; i=i+1)
       begin : port_a_out
	if (!(`PORT_IGNORE_COND) && (`PORT_IGNORE_OUTPUT_COND))
	    assign PIN[i] = PORTA[i];
	else if (!(`PORT_IGNORE_COND))
	  begin
	    assign PORTA[i] = DDR[i] ? PORT[i]
		       : (~DDR[i] & PORT[i]) ? PU[i]
		       : 1'bz;
	    assign PIN[i] = DDR[i] ? PORT[i] : PORTA[i];
	    assign PU[i] = 1'bZ;     // XXX not yet
	  end
       end
  `endif
  `ifdef IOR_HAVE_PORTB
    for (i=8; i<16; i=i+1)
       begin : port_b_out
	if (!(`PORT_IGNORE_COND) && (`PORT_IGNORE_OUTPUT_COND))
	    assign PIN[i] = PORTB[i-8];
	else if (!(`PORT_IGNORE_COND))
	  begin
	    assign PORTB[i-8] = DDR[i] ? PORT[i]
		       : (~DDR[i] & PORT[i]) ? PU[i]
		       : 1'bz;
	    assign PIN[i] = DDR[i] ? PORT[i] : PORTB[i-8];
	    assign PU[i] = 1'bZ;     // XXX not yet
	  end
       end
  `endif
  `ifdef IOR_HAVE_PORTC
    for (i=16; i<24; i=i+1)
       begin : port_c_out
	if (!(`PORT_IGNORE_COND) && (`PORT_IGNORE_OUTPUT_COND))
	    assign PIN[i] = PORTC[i-16];
	else if (!(`PORT_IGNORE_COND))
	  begin
	    assign PORTC[i-16] = DDR[i] ? PORT[i]
		       : (~DDR[i] & PORT[i]) ? PU[i]
		       : 1'bZ;
	    assign PIN[i] = DDR[i] ? PORT[i] : PORTC[i-16];
	    assign PU[i] = 1'bZ;     // XXX not yet
	  end
       end
  `endif
  endgenerate
`endif

endmodule
`endif
