/*
 * Copyright (C) 2011 Koji Suzuki
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.1 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

`include "rtavr_defs.v"
// global defines : OUTPUT_MUX_TRI 

//`define OUTPUT_MUX_NONE
//`define OUTPUT_MUX_TRI

`define ROM_USE_LATCH

// Module Name: rtavr_rom
//  ver 005
// 
// Target Device:   xc3s200a-4ft256
// Product Version: ISE 12.4
// Design Goal:     Balanced
//
//                   (1KW)       NORMAL MUX_TRI(FAKE) MUX_NONE  
// Number of Slice Flip Flops:      9          9         9    
// Number of 4 input LUTs:         10         10        10
// Number of occupied Slices:      10         10        10
// Total Number of 4 input LUTs:   10         10        10
// Number of bonded IOBs :         56         56        55
//   IOB Flip Flops :               -          -         -
// Number of BUFGMUXs :             1          1         1
// Number of RAMB16BWEs :           1          1         1
// Clock to Setup 
//on destination clock CLK          ?          ?         ?
// WARNING:Xst:647 - Input <ADDRA<11:10>> is never used. ..
// WARNING:Xst:646 - Signal <r_addrb<11:10>> is assigned but ..

//        ___        ______        ______
//  CLK2X    |______|      |______|      |______
//
//        _____               ______________
//  CLK        |_____________|              |___
//
// Normal                    |
//                         read/write
// CLK2X
//                   |            |
//                 read         write

module rtavr_rom # (
//		parameter SIZE = 1024
		parameter SIZE = 2048
//		parameter SIZE = 4096
) (
// for CPU 0
		input [11:0] ADDRA
		, output [15:0] DOA
`ifdef ROM_PSEUDO_DP
		, input OEA
`endif
		, input WEB
		, input [12:0] ADDRB
		, input [7:0] DIB
		, output[7:0] DOB
`ifdef OUTPUT_MUX_NONE
    `ifdef ROM_PSEUDO_DP
		, input OEB
    `endif
`else
		, input OEB
`endif
// for CPU 1
`ifdef ROM_PSEUDO_DP
    `ifdef ROM_HAVE_PORTCD
		, input OEC
		, input [11:0] ADDRC
		, output [15:0] DOC

		, input OED
		, input WED
		, input [12:0] ADDRD
		, input [7:0] DID
		, output[7:0] DOD
    `endif
`endif
	`ifdef SYS_DRIVEN_CLK4PH
		, input CLK_0
		, input CLK_90
		, input CLK_180
		, input CLK_270
	`else // SYS_DRIVEN_CLK2X
		, input CLK2X
		, input CLK
	`endif
		);

  reg  [15:0] rom [0:SIZE-1];

  `ifdef SYS_DRIVEN_CLK4PH
	wire CLK       = CLK_180;
	wire __CLK_180 = 1'b0;
	wire CLK_B = (~WEB & & CLK_0 & CLK_90) | (WEB & CLK_180 & CLK_270);
	wire __CLK_B = 1'b0;
     `ifdef ROM_HAVE_PORTCD
	wire CLK_D = (~WEB & & CLK_0 & CLK_90) | (WEB & CLK_180 & CLK_270);
	wire __CLK_D = 1'b0;
     `endif
  `else
	wire __CLK_180 = CLK;
	wire CLK_B = CLK2X;
	wire __CLK_B = ~CLK;
     `ifdef ROM_HAVE_PORTCD
	wire CLK_D = CLK2X;
	wire __CLK_D = ~CLK;
     `endif
  `endif

`ifdef ROM_PSEUDO_DP
    // PORTA,PORTB 
    wire [11:0] ADDR_AB = OEB ? ADDRB[12:1] : ADDRA;
    reg  [11:0] r_addrb;
    reg  r_dobh;
    reg  [7:0] r_tmpb;
    reg  [15:0] r_doa;
  
    wire WEBH = ADDRB[0] & WEB;
    wire WEBL = ~ADDRB[0] & WEB;
  
    wire [15:0] ROM_DO_AB = rom[r_addrb];

    always @(posedge CLK_B)
    begin
	if(WEBL & ~__CLK_B) r_tmpb <=  DIB;
	if(WEBH & ~ __CLK_B) rom[ADDR_AB] <=  { DIB , r_tmpb };
	r_addrb <= ADDR_AB;
	r_dobh <= ADDRB[0];
    end
  `ifdef OUTPUT_MUX_NONE
    assign DOB = (r_dobh)? ROM_DO_AB[15:8]: ROM_DO_AB[7:0];
  `elsif OUTPUT_MUX_TRI  
    bufif1(DOB, (r_dobh)? ROM_DO_AB[15:8] : ROM_DO_AB[7:0], OEB);
  `else 
    assign DOB = ~OEB ? 8'h00 : (r_dobh)? ROM_DO_AB[15:8]
				: ROM_DO_AB[7:0];
  `endif

  `ifdef ROM_USE_LATCH
    `ifdef SYS_DRIVEN_CLK4PH
	wire CLK_L     = CLK_180;
     `else
	wire CLK_L     = CLK;
     `endif

     always @(ROM_DO_AB or CLK_L)
     begin
	if(~CLK_L) r_doa <= ROM_DO_AB;
     end
     assign DOA = r_doa;
  `else
     always @(`posedge_CLK_180)
     begin
	if(OEA & ~__CLK_180) r_doa <= ROM_DO_AB;
     end
//     assign DOA = r_doa;
     assign DOA = CLK ? r_doa : ROM_DO_AB;
  `endif

`ifdef ROM_HAVE_PORTCD
    // PORTC,PORTD 
    wire [11:0] ADDR_CD = OED ? ADDRD[12:1] : ADDRC;
    reg  [11:0] r_addrd;
    reg   r_dodh;
    reg  [7:0] r_tmpd;
    reg  [15:0] r_doc;
  
    wire WEDH = ADDRD[0] & WED;
    wire WEDL = ~ADDRD[0] & WED;
  
    wire [15:0] ROM_DO_CD = rom[r_addrd];

    always @(posedge CLK_D)
    begin
	if(WEDL & ~__CLK_D) r_tmpd <=  DID;
	if(WEDH & ~ __CLK_D) rom[ADDR_CD] <=  { DID , r_tmpd };
	r_addrd <= ADDR_CD;
	r_dodh <= ADDRD[0];
    end
  `ifdef OUTPUT_MUX_NONE
    assign DOD = (r_dodh)? ROM_DO_CD[15:8]: ROM_DO_CD[7:0];
  `elsif OUTPUT_MUX_TRI  
    bufif1(DOD, (r_dodh)? ROM_DO_CD[15:8] : ROM_DO_CD[7:0], OED);
  `else 
    assign DOD = ~OED ? 8'h00 : (r_dodh)? ROM_DO_CD[15:8]
				: ROM_DO_CD[7:0];
  `endif

  `ifdef ROM_USE_LATCH
     always @(ROM_DO_CD or CLK_L)
     begin
	if(~CLK_L) r_doc <= ROM_DO_CD;
     end
     assign DOC = r_doc;
  `else
     always @(`posedge_CLK_180)
     begin
	if(OEC & ~__CLK_180) r_doc <= ROM_DO_CD;
     end
//     assign DOC = r_doc;
     assign DOC = CLK ? r_doc : ROM_DO_CD;
  `endif

`endif // ROM_HAVE_PORTCD
`else  // ROM_PSEUDO_DP

  reg  [11:0] r_addra;

  `ifdef SYS_DRIVEN_CLK4PH
	wire CLK_A = CLK_90;
	wire __CLK_A = 1'b0;
  `else
	wire CLK_A = CLK2X;
	wire __CLK_A = CLK;
  `endif

    always @(posedge CLK_A)
    begin
	if (~__CLK_A) r_addra <= ADDRA;
    end
    assign DOA = rom[r_addra];

    // PORTB 
    reg [11:0] r_addrb;
    reg r_doh;
    reg  [7:0] r_tmp;
  
    wire WEBH = ADDRB[0] & WEB;
    wire WEBL = ~ADDRB[0] & WEB;
  
    always @(posedge CLK_B)
    begin
	if(WEBL & ~__CLK_B) r_tmp <=  DIB;
	if(WEBH & ~ __CLK_B) rom[ADDRB[12:1]] <=  { DIB , r_tmp };
	r_addrb <= ADDRB[12:1];
	r_doh <= ADDRB[0];
    end
  `ifdef OUTPUT_MUX_NONE
    assign DOB = (r_doh)? rom[r_addrb][15:8]: rom[r_addrb][7:0];
  `elsif OUTPUT_MUX_TRI  
    bufif1(DOB, (r_doh)? rom[r_addrb][15:8] : rom[r_addrb][7:0], OEB);
  `else 
    assign DOB = ~OEB ? 8'h00 : (r_doh)? rom[r_addrb][15:8]
				: rom[r_addrb][7:0];
  `endif

`endif // ROM_PSEUDO_DP

//initial
//  begin : rom_data
//    rom[   0] = 16'h0000;
//  end

`ifdef ROM_INCLUDE_V
`include "rom_data.v"
`elsif ROM_INCLUDE_DAT
  initial
    begin 
       $readmemh("rom_data.mem" , rom, 0 , SIZE-1);
    end
`endif

endmodule
